/*
 * MusicBrainz -- The Internet music metadatabase
 *
 * Copyright (C) 2007 Lukas Lalinsky
 *  
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * $Id: tag.cpp 9216 2007-06-25 19:42:20Z luks $
 */
 
#include <string>
#include <musicbrainz3/tag.h>
#include "utils_private.h"

using namespace std;
using namespace MusicBrainz;

class Tag::TagPrivate
{
public:
	TagPrivate()
		{}
		
	string name;
	int count;
};

Tag::Tag(const string &name, int count)
{
	d = new TagPrivate();
	
	d->name = name;
	d->count = count;
}

Tag::~Tag()
{
	delete d;
}

SIMPLE_STRING_SETTER_GETTER(Tag, Name, name);
SIMPLE_INT_SETTER_GETTER(Tag, Count, count);
