/* $Id: utils_releasetypenames.h 7772 2006-05-31 18:53:04Z luks $ */

static const char *releaseTypeNames[][2] = {
	{"http://musicbrainz.org/ns/mmd-1.0#None", "None"},
	{"http://musicbrainz.org/ns/mmd-1.0#Album", "Album"},
	{"http://musicbrainz.org/ns/mmd-1.0#Single", "Single"},
	{"http://musicbrainz.org/ns/mmd-1.0#EP", "EP"},
	{"http://musicbrainz.org/ns/mmd-1.0#Compilation", "Compilation"},
	{"http://musicbrainz.org/ns/mmd-1.0#Soundtrack", "Soundtrack"},
	{"http://musicbrainz.org/ns/mmd-1.0#Spokenword", "Spokenword"},
	{"http://musicbrainz.org/ns/mmd-1.0#Interview", "Interview"},
	{"http://musicbrainz.org/ns/mmd-1.0#Audiobook", "Audiobook"},
	{"http://musicbrainz.org/ns/mmd-1.0#Live", "Live"},
	{"http://musicbrainz.org/ns/mmd-1.0#Remix", "Remix"},
	{"http://musicbrainz.org/ns/mmd-1.0#Other", "Other"},
	{"http://musicbrainz.org/ns/mmd-1.0#Official", "Official"},
	{"http://musicbrainz.org/ns/mmd-1.0#Promotion", "Promotion"},
	{"http://musicbrainz.org/ns/mmd-1.0#Bootleg", "Bootleg"},
};

