/* $Id: utils_scriptnames.h 7772 2006-05-31 18:53:04Z luks $ */

static const char *scriptNames[][2] = {
	{"Yiii", "Yi"},
	{"Telu", "Telugu"},
	{"Taml", "Tamil"},
	{"Guru", "Gurmukhi"},
	{"Hebr", "Hebrew"},
	{"Geor", "Georgian (Mkhedruli)"},
	{"Ugar", "Ugaritic"},
	{"Cyrl", "Cyrillic"},
	{"Hrkt", "Kanji & Kana"},
	{"Armn", "Armenian"},
	{"Runr", "Runic"},
	{"Khmr", "Khmer"},
	{"Latn", "Latin"},
	{"Hani", "Han (Hanzi, Kanji, Hanja)"},
	{"Ital", "Old Italic (Etruscan, Oscan, etc.)"},
	{"Hano", "Hanunoo (Hanunóo)"},
	{"Ethi", "Ethiopic (Ge'ez)"},
	{"Gujr", "Gujarati"},
	{"Hang", "Hangul"},
	{"Arab", "Arabic"},
	{"Thaa", "Thaana"},
	{"Buhd", "Buhid"},
	{"Sinh", "Sinhala"},
	{"Orya", "Oriya"},
	{"Hans", "Han (Simplified variant)"},
	{"Thai", "Thai"},
	{"Cprt", "Cypriot"},
	{"Linb", "Linear B"},
	{"Hant", "Han (Traditional variant)"},
	{"Osma", "Osmanya"},
	{"Mong", "Mongolian"},
	{"Deva", "Devanagari (Nagari)"},
	{"Laoo", "Lao"},
	{"Tagb", "Tagbanwa"},
	{"Hira", "Hiragana"},
	{"Bopo", "Bopomofo"},
	{"Goth", "Gothic"},
	{"Tale", "Tai Le"},
	{"Mymr", "Myanmar (Burmese)"},
	{"Tglg", "Tagalog"},
	{"Grek", "Greek"},
	{"Mlym", "Malayalam"},
	{"Cher", "Cherokee"},
	{"Tibt", "Tibetan"},
	{"Kana", "Katakana"},
	{"Syrc", "Syriac"},
	{"Cans", "Unified Canadian Aboriginal Syllabics"},
	{"Beng", "Bengali"},
	{"Limb", "Limbu"},
	{"Ogam", "Ogham"},
	{"Knda", "Kannada"},
};

